---
title: "Log service"
---

Log service allows clients to log events

#### List
Returns a list of log entries generated by a specified source id. Each log entry contains an "id" field suitable to be used as a continuation token for the optional "after" argument.

Arguments:
- `src`: ID of an entity that generated log entries that you want to fetch.
- `after`: Continuation token. Put here an `id` of the last entry returned by the previous call.
- `limit`: Maximum number of entries to return.
- `stream`: Log stream name.

Result `array`: 

Definition:
```json
{
  "doc": "Returns a list of log entries generated by a specified source id. Each log entry contains an \"id\" field suitable to be used as a continuation token for the optional \"after\" argument.",
  "args": {
    "src": {
      "doc": "ID of an entity that generated log entries that you want to fetch.",
      "type": "string"
    },
    "after": {
      "doc": "Continuation token. Put here an `id` of the last entry returned by the previous call.",
      "type": "string"
    },
    "limit": {
      "doc": "Maximum number of entries to return.",
      "type": "integer"
    },
    "stream": {
      "doc": "Log stream name.",
      "type": "string"
    }
  },
  "required_args": [
    "src"
  ],
  "result": {
    "items": {
      "type": "object",
      "properties": {
        "msg": {
          "type": [
            "object",
            "array",
            "string"
          ],
          "keep_as_json": true
        },
        "timestamp": {
          "type": "integer"
        },
        "id": {
          "type": "string"
        },
        "stream": {
          "type": "string"
        }
      }
    },
    "type": "array"
  }
}
```

Request:
```json
{
  "v": 1,
  "src": "//api.cesanta.com/user_123",
  "dst": "//api.cesanta.com",
  "cmds": [
    {
      "cmd": "/v1/Log.List",
      "id": 123,
      "args": {
        "src": "//api.cesanta.com/device_123",
        "limit": 10,
        "stream": "console"
      }
    }
  ]
}

```

Response:
```json
{
  "v": 1,
  "src": "//api.cesanta.com",
  "dst": "//api.cesanta.com/user_123",
  "resp": [
    {
      "id": 123,
      "status": 0,
      "resp": [
        {
          "stream": "console",
          "msg": "hello world!",
          "timestamp": 1453395062123,
          "id": "msg123"
        }
      ]
    }
  ]
}

```

#### Log
Records a log entry associated with the src id. If not provided stream defaults to "console". Timestamp is server generated if omitted, otherwise is milliseconds from epoch UTC.

Arguments:
- `msg`: Message to log.
- `src`: Override src id. Used to log on behalf of another source, if permitted.
- `stream`: Log stream name. Defaults to `"console"`.
- `timestamp`: Timestamp of a message in milliseconds since Epoch.


Definition:
```json
{
  "doc": "Records a log entry associated with the src id. If not provided stream defaults to \"console\". Timestamp is server generated if omitted, otherwise is milliseconds from epoch UTC.",
  "args": {
    "msg": {
      "doc": "Message to log.",
      "type": [
        "object",
        "array",
        "string"
      ],
      "keep_as_json": true
    },
    "src": {
      "doc": "Override src id. Used to log on behalf of another source, if permitted.",
      "type": "string"
    },
    "stream": {
      "doc": "Log stream name. Defaults to `\"console\"`.",
      "type": "string"
    },
    "timestamp": {
      "doc": "Timestamp of a message in milliseconds since Epoch.",
      "type": "integer"
    }
  },
  "required_args": [
    "msg"
  ]
}
```

Request:
```json
{
  "v": 1,
  "src": "//api.cesanta.com/device_123",
  "dst": "//api.cesanta.com",
  "cmds": [
    {
      "cmd": "/v1/Log.Log",
      "id": 123,
      "args": {
        "msg": "hello world!"
      }
    }
  ]
}

```

Response:
```json
{
  "v": 1,
  "src": "//api.cesanta.com",
  "dst": "//api.cesanta.com/device_123",
  "resp": [
    {
      "id": 123,
      "status": 0
    }
  ]
}
```


